/*
	AppController.m
	Copyright (c) 2001 by Apple Computer, Inc., all rights reserved.
	Author: Chuck Pisula

	Modified by ST 010928
*/


#import "AppController.h"
#import "FSBrowser.h"
#import "FSNodeInfo.h"
#import "FSBrowserCell.h"
#import "Utils.h"
#import "MailTitles.h"
#import "MailContents.h"
#import "Preferences.h"
#import "MenuController.h"

@interface AppController (PrivateUtilities)
- (NSString*)fsPathToColumn:(int)column;
@end


@implementation AppController

- (void)awakeFromNib
{
NSString *initIdentifier = 0;
NSString *sortFolderTag;
    
	[[NSNotificationCenter defaultCenter] addObserver: self
		selector: @selector(terminate:)
		name: NSApplicationWillTerminateNotification object:nil];

    [fsBrowser setFrame: [[mailFolderWindow contentView] frame]];
    [fsBrowser setCellClass: [FSBrowserCell class]];

    //[fsBrowser setTarget: self];
    //[fsBrowser setAction: @selector(browserSingleClick:)];
    //[fsBrowser setDoubleAction: @selector(browserDoubleClick:)];
    
    [fsBrowser setMaxVisibleColumns: 3];
    [fsBrowser setMinColumnWidth: 110];
		
	[mailTitleScrollView setFrame: [[mailTitleWindow contentView] frame]];

	[mailContents registerAndReplaceTextView];
	
	[mailTitleWindow makeFirstResponder: mailTitleScrollView];
	[mailTitleScrollView setNextResponder: (NSResponder *)[mailContents
			browserTextView]];
	
	[mailFolderWindow makeFirstResponder: fsBrowser];
	[fsBrowser setNextResponder: (NSResponder *)[mailContents
			browserTextView]];
	
	if (SetOMEPath())
		exit(-1);
    
    [mailTitles setReverse: [[NSUserDefaults standardUserDefaults]
                        boolForKey: @"Sort Reverse"]];
    
    if (!(initIdentifier = [[NSUserDefaults standardUserDefaults]
                            stringForKey: @"Sort Identifier"]))
        initIdentifier = @"date";
    [mailTitles setIdentifier: initIdentifier];
    
    if (!(sortFolderTag = [[NSUserDefaults standardUserDefaults]
                            stringForKey: @"Sort Folder by"]))
        sortFolderTag = @"name";
    [menuController sortFolderByTag: sortFolderTag];
    
	[self browseInBox];
//	[self browseDraft];
	
	[preferences loadPrefAndStartUpDownload];
}


- (void) terminate: (NSNotification *) notification
{
	[[NSUserDefaults standardUserDefaults]
           setBool: [mailTitles reverse] forKey: @"Sort Reverse"];
	[[NSUserDefaults standardUserDefaults]
           setObject: [mailTitles identifier] forKey: @"Sort Identifier"];
	[[NSUserDefaults standardUserDefaults]
           setObject: [FSNodeInfo sortFolderTag] forKey: @"Sort Folder by"];
}

- (void) browseInBox
{
char dirPath[1024];

	sprintf(dirPath, "%s/%s/", omeRootPath, "InBox");
	[fsBrowser setPath:	[NSString stringWithUTF8String: dirPath]];
	[fsBrowser scrollColumnToVisible: [fsBrowser lastColumn]];
	[fsBrowser scrollColumnsRightBy: 1];
	
	[mailTitles loadTitleTable: dirPath refresh: YES];
	[mailTitleWindow setTitle: @"InBox"];
}

- (void) browseDraft
{
char dirPath[1024];

	sprintf(dirPath, "%s/%s/", omeRootPath, "Draft");
	[fsBrowser setPath:	[NSString stringWithUTF8String: dirPath]];
	[fsBrowser scrollColumnToVisible: [fsBrowser lastColumn]];
	[fsBrowser scrollColumnsRightBy: 1];
	
	[mailTitles loadTitleTable: dirPath refresh: YES];
	[mailTitleWindow setTitle: @"Draft"];
}

// ==========================================================
// Browser Delegate Methods.
// ==========================================================

// Use lazy initialization, since we don't want to touch the file system too much.
- (int)browser:(NSBrowser *)sender numberOfRowsInColumn:(int)column {
    NSString   *fsNodePath = nil;
    FSNodeInfo *fsNodeInfo = nil;
    
    // Get the absolute path represented by the browser selection, and create a fsnode for the path.
    // Since column represents the column being (lazily) loaded fsNodePath is the path for the last selected cell.
    fsNodePath = [self fsPathToColumn: column];
    fsNodeInfo = [FSNodeInfo nodeWithParent: nil atRelativePath: fsNodePath];
    
    return [[fsNodeInfo visibleSubNodes] count];
}

- (void)browser:(NSBrowser *)sender willDisplayCell:(id)cell atRow:(int)row column:(int)column {
    NSString   *containingDirPath = nil;
    FSNodeInfo *containingDirNode = nil;
    FSNodeInfo *displayedCellNode = nil;
    NSArray    *directoryContents = nil;
    
    // Get the absolute path represented by the browser selection, and create a fsnode for the path.
    // Since (row,column) represents the cell being displayed, containingDirPath is the path to it's containing directory.
    containingDirPath = [self fsPathToColumn: column];
    containingDirNode = [FSNodeInfo nodeWithParent: nil atRelativePath: containingDirPath];
    
    // Ask the parent for a list of visible nodes so we can get at a FSNodeInfo for the cell being displayed.
    // Then give the FSNodeInfo to the cell so it can determine how to display itself.
    directoryContents = [containingDirNode visibleSubNodes];
    displayedCellNode = [directoryContents objectAtIndex: row];
    
    [cell setStringValueFromFSNodeInfo: displayedCellNode];
}

// ==========================================================
// Browser Target / Action Methods.
// ==========================================================

- (IBAction)browserSingleClick:(NSBrowser *)browser {
	[mailTitleWindow setTitle: [[browser path] lastPathComponent]];
	[mailTitles loadTitleTable: (char *)[[browser path] fileSystemRepresentation] refresh: YES];
	[browser scrollColumnsRightBy: 1];
}

- (IBAction)browserDoubleClick:(id)browser { /* Does not work in Tiger */
 	[browser scrollColumnsLeftBy: 1];
	//[browser scrollColumnsRightBy: 1];
 	//[browser scrollColumnsLeftBy: 2];
}


@end

@implementation AppController (PrivateUtilities)

- (NSString*)fsPathToColumn:(int)column {
    NSString *path = nil;
    if(column==0) path = [NSString stringWithFormat: @"/"];
    else path = [fsBrowser pathToColumn: column];
    return path;
}

@end



