//
//  FSNodeInfo.h
//
//  Copyright (c) 2001 Apple. All rights reserved.
//
//  FSNodeInfo encapsulates information about a file or directory.
//  This implementation is not necessarily the best way to do something like this,
//  it is simply a wrapper to make the rest of the browser code easy to follow.

#import <Cocoa/Cocoa.h>
@class AppController;

@interface FSNodeInfo : NSObject {
@private
    NSString    *relativePath;  // Path relative to the parent.
    FSNodeInfo  *parentNode;	// Containing directory, not retained to avoid retain/release cycles.
    NSDate *modDate;
}

+ (FSNodeInfo *)nodeWithParent:(FSNodeInfo*)parent atRelativePath:(NSString *)path;

+ (NSString *) sortFolderTag;
+ (void) setSortFolderTag: (NSString *) newSortFolderTag;

- (id)initWithParent:(FSNodeInfo*)parent atRelativePath:(NSString*)path;

- (void)dealloc;

- (NSArray *)subNodes;
- (NSArray *)visibleSubNodes;

- (NSString *)absolutePath;
- (NSString *)lastPathComponent;

- (BOOL)isDirectory;

- (BOOL)isVisible;

- (NSComparisonResult) compareByDate: (FSNodeInfo *) targetNode;
- (NSDate *) modDate;
- (void) setModDate: (NSDate *) newModDate;

@end
