/*
	FSNodeInfo.m
	Copyright (c) 2001 by Apple Computer, Inc., all rights reserved.
	Author: Chuck Pisula

	Modified by ST 010921
*/

#import "FSNodeInfo.h"
#import "AppController.h"

@implementation FSNodeInfo 

static NSString *sortFolderTag = @"name";

+ (FSNodeInfo*)nodeWithParent:(FSNodeInfo*)parent atRelativePath:(NSString *)path {
    return [[[FSNodeInfo alloc] initWithParent:parent atRelativePath:path] autorelease];
}


+ (NSString *) sortFolderTag
{
    return sortFolderTag;
}


+ (void) setSortFolderTag: (NSString *) newSortFolderTag
{
    [sortFolderTag autorelease];
    sortFolderTag = [newSortFolderTag copy];
}


- (id)initWithParent:(FSNodeInfo*)parent atRelativePath:(NSString*)path {    
    self = [super init];
    if (self==nil) return nil;
    
    parentNode = parent;
    relativePath = [path retain];
    
    modDate = (NSDate *) [[[NSFileManager defaultManager] fileAttributesAtPath:
                [self absolutePath] traverseLink: YES]
                objectForKey: NSFileModificationDate];

    return self;
}

- (void)dealloc {
    // parentNode is not released since we never retained it.
    [relativePath release];
    relativePath = nil;
    parentNode = nil;
    [super dealloc];
}

- (NSArray *)subNodes {
    NSString       *subNodePath = nil;
    NSEnumerator   *subNodePaths = [[[NSFileManager defaultManager]
                        directoryContentsAtPath: [self absolutePath]] objectEnumerator];
    NSMutableArray *subNodes = [NSMutableArray array];
    
    while ((subNodePath=[subNodePaths nextObject])) {
        FSNodeInfo *node = [FSNodeInfo nodeWithParent:self atRelativePath: subNodePath];
		if ([node isDirectory])
			[subNodes addObject: node];
    }
    return subNodes;
}

#if 1 /*.Trash or .vol should not be seen*/
- (NSArray *)visibleSubNodes {
    FSNodeInfo     *subNode = nil;
    NSEnumerator   *allSubNodes = [[self subNodes] objectEnumerator];
    NSMutableArray *visibleSubNodes = [NSMutableArray array];
    
    while ((subNode=[allSubNodes nextObject])) {
        if ([subNode isVisible]) [visibleSubNodes addObject: subNode];
    }
    
#if 1
    if ([sortFolderTag isEqualToString: @"modDate"] && [visibleSubNodes count])
        [visibleSubNodes sortUsingSelector: @selector(compareByDate:)];
#endif
    return visibleSubNodes;
}
#endif

- (BOOL)isDirectory {
    BOOL isDir = NO;
    BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:[self absolutePath] isDirectory:&isDir];
    return (exists && isDir);
}

#if 1
- (BOOL)isVisible {
    // Make this as sophisticated for example to hide more files you don't think the user should see!
    NSString *lastPathComponent = [self lastPathComponent];
    return ([lastPathComponent length] ? ([lastPathComponent characterAtIndex:0]!='.') : NO);
}
#endif


- (NSString*)lastPathComponent {
    return [relativePath lastPathComponent];
}


- (NSString*)absolutePath {
    NSString *result = relativePath;
    if(parentNode!=nil) {
        NSString *parentAbsPath = [parentNode absolutePath];
        if ([parentAbsPath isEqualToString: @"/"]) parentAbsPath = @"";
        result = [NSString stringWithFormat: @"%@/%@", parentAbsPath, relativePath];
    }
    return result;
}


- (NSComparisonResult) compareByDate: (FSNodeInfo *) targetNode
{
    return - [[self modDate] compare: [targetNode modDate]];
}


- (NSDate *) modDate
{
    return modDate;
}


- (void) setModDate: (NSDate *) newModDate
{
    modDate = newModDate;
}

@end
